
`resetall

module secure_fpga_template (
  JTAG_7,       //swdio
  JTAG_9,       //swclk
  UART0RXD,
  UART0TXD,
  HCLK,		    //System Clock
  hwRstn	    //System Reset
);

input HCLK;     // System clock
input hwRstn;   // System reset

//UART0
input UART0RXD;
output UART0TXD;

//SWD
inout JTAG_7;
inout JTAG_9;

wire MCLK;          //mcu clock

//pll
Gowin_rPLL u_Gowin_rPLL
(
  .clkout(MCLK),    //output clkout 20MHz
  .clkin(HCLK)      //input clkin 50MHz
);

wire [31:0] EXTFLASH0HRDATA;
wire EXTFLASH0HREADYOUT;
wire [1:0] EXTFLASH0HRESP;
wire [1:0] EXTFLASH0HTRANS;
wire [2:0] EXTFLASH0HBURST;
wire [3:0] EXTFLASH0HPROT;
wire [2:0] EXTFLASH0HSIZE;
wire EXTFLASH0HWRITE;
wire EXTFLASH0HREADYMUX;
wire [3:0] EXTFLASH0HMASTER;
wire EXTFLASH0HMASTLOCK;
wire [31:0] EXTFLASH0HADDR;
wire [31:0] EXTFLASH0HWDATA;
wire EXTFLASH0HSEL;
wire EXTFLASH0HCLK;
wire EXTFLASH0HRESET;

//userflash controller
GW_FLASH_gowin_top u_GW_FLASH_gowin_top (
    .hclk                   (EXTFLASH0HCLK),
    .hresetn                (EXTFLASH0HRESET),
    .targflash0_hsel_i      (EXTFLASH0HSEL),
    .targflash0_haddr_i     (EXTFLASH0HADDR),
    .targflash0_htrans_i    (EXTFLASH0HTRANS),
    .targflash0_hwrite_i    (EXTFLASH0HWRITE),
    .targflash0_hsize_i     (EXTFLASH0HSIZE),
    .targflash0_hburst_i    (EXTFLASH0HBURST),
    .targflash0_hprot_i     (EXTFLASH0HPROT),
    .targflash0_memattr_i   (2'b00),
    .targflash0_exreq_i     (1'b0),
    .targflash0_hmaster_i   (EXTFLASH0HMASTER),
    .targflash0_hwdata_i    (EXTFLASH0HWDATA),
    .targflash0_hmastlock_i (EXTFLASH0HMASTLOCK),
    .targflash0_hreadymux_i (EXTFLASH0HREADYMUX),
    .targflash0_hauser_i    (1'b0),
    .targflash0_hwuser_i    (4'b0000),
    .targflash0_hrdata_o    (EXTFLASH0HRDATA),
    .targflash0_hreadyout_o (EXTFLASH0HREADYOUT),
    .targflash0_hresp_o     (EXTFLASH0HRESP),
    .targflash0_exresp_o    (),
    .targflash0_hruser_o    (),
    .flash_err_o            (),    // Flash memory/System error
    .flash_int_o            (),    // EFlash interrupt

    .pclk                   (EXTFLASH0HCLK),
    .pclkg                  (EXTFLASH0HCLK)
);

//gowin empu m1
Gowin_EMPU_M1_Top u_Gowin_EMPU_M1_Top (
  .LOCKUP(),
  .HALTED(),
  .JTAG_3(),
  .JTAG_4(),
  .JTAG_5(),
  .JTAG_6(),
  .JTAG_7(JTAG_7),
  .JTAG_8(),
  .JTAG_9(JTAG_9),
  .JTAG_10(),
  .JTAG_11(),
  .JTAG_12(),
  .JTAG_13(),
  .JTAG_14(),
  .JTAG_15(),
  .JTAG_16(),
  .JTAG_17(),
  .JTAG_18(),
  .UART0RXD(UART0RXD),
  .UART0TXD(UART0TXD),
  .EXTFLASH0HRDATA(EXTFLASH0HRDATA), //input [31:0] EXTFLASH0HRDATA
  .EXTFLASH0HREADYOUT(EXTFLASH0HREADYOUT), //input EXTFLASH0HREADYOUT
  .EXTFLASH0HRESP(EXTFLASH0HRESP), //input [1:0] EXTFLASH0HRESP
  .EXTFLASH0HTRANS(EXTFLASH0HTRANS), //output [1:0] EXTFLASH0HTRANS
  .EXTFLASH0HBURST(EXTFLASH0HBURST), //output [2:0] EXTFLASH0HBURST
  .EXTFLASH0HPROT(EXTFLASH0HPROT), //output [3:0] EXTFLASH0HPROT
  .EXTFLASH0HSIZE(EXTFLASH0HSIZE), //output [2:0] EXTFLASH0HSIZE
  .EXTFLASH0HWRITE(EXTFLASH0HWRITE), //output EXTFLASH0HWRITE
  .EXTFLASH0HREADYMUX(EXTFLASH0HREADYMUX), //output EXTFLASH0HREADYMUX
  .EXTFLASH0HMASTER(EXTFLASH0HMASTER), //output [3:0] EXTFLASH0HMASTER
  .EXTFLASH0HMASTLOCK(EXTFLASH0HMASTLOCK), //output EXTFLASH0HMASTLOCK
  .EXTFLASH0HADDR(EXTFLASH0HADDR), //output [31:0] EXTFLASH0HADDR
  .EXTFLASH0HWDATA(EXTFLASH0HWDATA), //output [31:0] EXTFLASH0HWDATA
  .EXTFLASH0HSEL(EXTFLASH0HSEL), //output EXTFLASH0HSEL
  .EXTFLASH0HCLK(EXTFLASH0HCLK), //output EXTFLASH0HCLK
  .EXTFLASH0HRESET(EXTFLASH0HRESET), //output EXTFLASH0HRESET
  .HCLK(MCLK),	    //System Clock
  .hwRstn(hwRstn)	//System Reset
);

endmodule