/*
 * Copyright 2021 Intrinsic ID B.V.  All rights reserved.
 *
 * Usage of this software is permitted under a valid written license agreement
 * between you and Intrinsic ID B.V.
 */

/*!
 * \file    iid_target.c
 * \brief   Hardware abstraction layer implementation.
 */

#include <stdarg.h>

#include "iid_target.h"

/*******************************************************************************
 * Target specific includes
 ******************************************************************************/
/* TODO: insert target specific include files here. */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define PRINTF_BUF_SIZE				32
#define FLAGS_ZEROPAD				0x01
#define FLAGS_UPPERCASE				0x02

/* TODO: insert target specific defines here. */
#define PLATFORM_UART				UART0

/*******************************************************************************
 * Types
 ******************************************************************************/
/* TODO: insert target specific types here. */

/*******************************************************************************
 * Variables
 ******************************************************************************/

/*******************************************************************************
 * Prototypes
 ******************************************************************************/

/*******************************************************************************
 * Code
 ******************************************************************************/
/* TODO: implement function for selected target. */
void target_init(void) {
	UART_InitTypeDef UART_InitStruct = {
		.UART_Mode.UARTMode_Tx = ENABLE,
		.UART_Mode.UARTMode_Rx = ENABLE,
		.UART_Int.UARTInt_Tx = DISABLE,
		.UART_Int.UARTInt_Rx = DISABLE,
		.UART_Ovr.UARTOvr_Tx = DISABLE,
		.UART_Ovr.UARTOvr_Rx = DISABLE,
		.UART_Hstm = DISABLE,
		.UART_BaudRate = 115200 //Baud Rate
	};

	SystemInit();				//Initializes system

	UART_Init(PLATFORM_UART, &UART_InitStruct);

}

/* TODO: implement function for selected target. */
void target_led_control(e_target_hw_led led, bool on) {
	switch(led) {
	case led_red:
		if (on) {

		} else {

		}
		break;

	case led_green:
		if (on) {

		} else {

		}
		break;

	case led_blue:
		if (on) {

		} else {

		}
		break;
	}
}

/* TODO: Replace XXX with funcion to read from the designated comm channel. */
char target_get_upper_case_char(void) {
	char input;

	while (1) {
		/* Read a character from the comm channel */
		input = UART_ReceiveChar(PLATFORM_UART);

		/* Skip every character smaller than 'A' or larger than 'Z' */
		if (('A' > input) || ('Z' < input)) {
			continue;
		}

		return input;
	}
}

/* TODO: implement function for selected target. */
char target_input_char(void){
	char input;

	/* Read a character from the comm channel */
	input = UART_ReceiveChar(PLATFORM_UART);

	return input;
}

/* TODO: implement function for selected target. */
void target_output_char(char value) {
	UART_SendChar(PLATFORM_UART, value);
}

/* TODO: implement function for selected target. */
void target_nvm_clear(void) {
}

/* TODO: implement function for selected target. */
void target_nvm_write(const uint32_t * src_addr, uint32_t num_bytes) {
}

/* TODO: implement function for selected target. */
void target_nvm_read(uint32_t * dest_addr, uint32_t num_bytes) {
}

/* TODO: modify function for selected target if required. */
void target_nvic_reset(void) {
	volatile int i;

	NVIC_SystemReset();

	for (;;) {
		i += 1;
	}
}

int target_printf(const char * restrict format, ...) {
	va_list va;
	char buf[PRINTF_BUF_SIZE];
	unsigned int base;
	unsigned int flags;
	unsigned int width;
	int value;
	int len;
	bool valid_flag;
	char digit;
	char * string;
	int idx = 0;

	va_start(va, format);

	while(*format) {
		/* Accepted format specifier :  %[flags][width]specifier */
		if ('%' != *format) {
			target_output_char(*format);
			idx++;
			format++;
			continue;
		} else {
			format++;
		}

		/* Evaluate the flags if present. Only the zero pad flag is accepted */
		flags = 0;
		do {
			switch (*format) {
			case '0':
				flags |= FLAGS_ZEROPAD;
				format++;
				valid_flag = true;
				break;
			default:
				valid_flag = false;
				break;
			}
		} while (valid_flag);

		/* Evaluate the width field */
		width = 0;
		while (('0' <= *format) && ('9' >= *format)) {
			width = (width * 10) + *format - '0';
			format++;
		}

		/* Evaluate the type specifier */
		switch (*format) {
		case 'd':
		case 'x':
		case 'X':
			if (*format == 'x' || *format == 'X') {
				base = 16;
			} else {
				base = 10;
			}

			if ('X' == *format) {
				flags |= FLAGS_UPPERCASE;
			}

			value = va_arg(va, int);
			len = 0;
			do {
				digit = (char)(value % base);
				buf[len++] = (10 > digit) ? '0' + digit : (flags & FLAGS_UPPERCASE ? 'A' : 'a') + digit - 10;
				value /= base;
			} while (value && (len < PRINTF_BUF_SIZE));

			if (flags & FLAGS_ZEROPAD) {
				while ((len < width) && (PRINTF_BUF_SIZE > len)) {
					buf[len++] = '0';
				}
			}

			while (len) {
				target_output_char(buf[--len]);
				idx++;
			}

			format++;
			break;

		case 'c':
			target_output_char(va_arg(va, int));
			idx++;
			format++;
			break;

		case 's':
			string = va_arg(va, char *);

			while (0 != *string) {
				target_output_char(*string);
				string++;
				idx++;
			}

			format++;
			break;

		default :
			target_output_char(*format);
			idx++;
			format++;
			break;
		}
	}

	va_end(va);

	return idx;
}

void print_hex_string(uint8_t * const hex_data, size_t length)
{
	int i;

	for (i = 0 ; i < length; i++)
	{
		PRINTF("%02X ", hex_data[i]);

		if (((0 != i) && (0 == (i + 1) % 16)) || (length == (i + 1)))
		{
			PRINTF("\r\n");
		}
	}
}
