/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 *
 * @file			main.c
 * @author		Embedded Development Team
 * @version		V1.6.1
 * @date			2021-01-01 09:00:00
 * @brief			Main function.
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <string.h>

#include "iidbroadkey.h"
#include "iid_target.h"		/* includes GOWIN_M1.h */

#include "bk_demo_app.h"

/* Macros -------------------------------------------------------------------*/
/*
 * Enable the following define to execute
 * A known answer text. This test loads the
 * PUF SRAM with known values as well as the
 * activation code.
 */
//#define BK_KNOWN_ANSWER_TEST

/*
 * Enable the following define to store a given
 * activation code in flash at location the
 * activation code should be stored. This define
 * is used without having to explicitly program the
 * activation code in flash separately.
 *
 * When using this define an activation code MUST previously
 * have been generated by the enroll/manufacture program.
 * The activation code generated by the enroll should be copied
 * into the board_ac array. The linker file has been setup
 * to store the board_ac array at the location of the AC in flash.
 */
//#define BK_AC_FLASH_DATA

#if defined(BK_KNOWN_ANSWER_TEST) && defined(BK_AC_FLASH_DATA)
#error BK_KNOWN_ANSWER_TEST and BK_AC_FLASH_DATA are mutually exclusive defines
#endif

//#define DEBUG_DUMP_PUF_SRAM

/*
 * Enable the following define to simplify debugging
 * Execution will halt in while loop until user
 * uses debugger to change test_flag value to FALSE
 */
//#define DEBUG_WAIT_LOOP


/* Imports ------------------------------------------------------------------*/
extern int __puf;	/* define in GOWIN_M1_flash.ld PUF address      */

#ifndef BK_KNOWN_ANSWER_TEST
extern int __ac; 	/* define in GOWIN_M1_flash.ld AC_FLASH address */
#endif

/* Declarations --------------------------------------------------------------*/

#ifdef BK_KNOWN_ANSWER_TEST
static const uint8_t local_sram[BK_SRAM_PUF_SIZE_BYTES] =
{
	0x45, 0xDD, 0x4F, 0x9A, 0xE5, 0xAF, 0xD0, 0xA3, 0xE9, 0xC3, 0x53, 0xFD, 0xF0, 0x3C, 0x89, 0xF0,
	0x85, 0x23, 0x1D, 0x97, 0xBE, 0xB6, 0x7A, 0x0B, 0x4D, 0x56, 0x4D, 0x05, 0x73, 0x2E, 0x65, 0x5E,
	0x65, 0x5D, 0x47, 0x44, 0xE5, 0xAF, 0xD0, 0xA7, 0xBC, 0xEE, 0xF9, 0xFD, 0xBB, 0x2E, 0xFD, 0xF0,
	0x79, 0xFB, 0x4E, 0x73, 0xEB, 0xBF, 0x0E, 0xF1, 0xBF, 0xD9, 0x37, 0x79, 0x92, 0x57, 0x4A, 0x27,
	0xE5, 0xDD, 0x47, 0x3E, 0xE5, 0xAF, 0xD2, 0xBC, 0xED, 0xEE, 0xF1, 0xFD, 0xBB, 0x6E, 0xED, 0xF2,
	0x7C, 0xFE, 0x5A, 0x6F, 0x7B, 0x27, 0xF2, 0xC3, 0xFF, 0x6F, 0x68, 0x73, 0xC5, 0xF2, 0x49, 0xD0,
	0x65, 0xD5, 0x47, 0xB6, 0xE5, 0xAF, 0xD8, 0xBC, 0x73, 0xEE, 0xF9, 0xFD, 0xAA, 0xEE, 0xFD, 0xB2,
	0x09, 0xD2, 0x19, 0xFF, 0xE5, 0x5A, 0xE2, 0x63, 0xA2, 0x6F, 0xEC, 0x33, 0x18, 0xF2, 0x48, 0xB7,
	0x67, 0x33, 0x35, 0x07, 0xB4, 0x54, 0xDB, 0x28, 0xAE, 0x54, 0xF5, 0x6F, 0x34, 0xE5, 0xFD, 0x3C,
	0x8F, 0xA8, 0x5A, 0x7D, 0x6F, 0x4C, 0xE2, 0x45, 0xFD, 0x6F, 0xE8, 0xBE, 0xD2, 0xF2, 0x48, 0xC1,
	0x61, 0xBB, 0x57, 0xC8, 0xF7, 0xD6, 0xD8, 0x7D, 0xED, 0xEE, 0xFB, 0xF5, 0xAB, 0xEE, 0xFD, 0xF0,
	0x6D, 0xFB, 0x5A, 0xF5, 0x4B, 0xFF, 0xE2, 0x0F, 0xDD, 0x6F, 0xC9, 0x96, 0xBA, 0xF6, 0x49, 0xFA,
	0x01, 0xB9, 0x08, 0x83, 0x28, 0xEE, 0x7F, 0x78, 0xED, 0xC1, 0xF9, 0x67, 0xAB, 0x7E, 0xFD, 0x30,
	0x7D, 0xFB, 0x5A, 0xAA, 0x4F, 0xFF, 0xE2, 0xAF, 0xF1, 0x4F, 0xE8, 0x35, 0xB6, 0x16, 0x48, 0x78,
	0x4D, 0xDD, 0x45, 0x36, 0xAD, 0xAF, 0xD0, 0xB4, 0x1E, 0xD7, 0xB7, 0x75, 0xD6, 0x1F, 0xFC, 0x66,
	0xCB, 0x5A, 0x01, 0x76, 0x92, 0xC8, 0x78, 0x1C, 0xFB, 0x6F, 0xE8, 0xD5, 0xC4, 0xF2, 0x48, 0xFD,
	0x65, 0x9D, 0x47, 0xE6, 0xE5, 0xAF, 0x90, 0xD5, 0x0B, 0xAE, 0xE1, 0x9D, 0xA0, 0xEE, 0x93, 0xF4,
	0x0C, 0xE9, 0x9E, 0xF7, 0x49, 0xB3, 0x04, 0x27, 0x3C, 0x6F, 0xE8, 0xD5, 0xCA, 0xB2, 0x48, 0x7D,
	0x45, 0xDD, 0x47, 0x6B, 0xE5, 0xAF, 0xD0, 0xB7, 0xED, 0x6E, 0xF9, 0xE5, 0xAB, 0xEE, 0xF5, 0x35,
	0xCD, 0x30, 0xF5, 0xD5, 0xA1, 0xEA, 0xD4, 0xF5, 0xFF, 0x6F, 0xE8, 0xD5, 0x92, 0xB2, 0x48, 0xFD,
	0x72, 0xA4, 0x1D, 0x9A, 0x5D, 0xD9, 0x40, 0xEC, 0xBE, 0x41, 0xD4, 0x54, 0x27, 0x6D, 0xB6, 0x0B,
	0x7C, 0xFB, 0x5A, 0x43, 0x6F, 0xFF, 0xE2, 0xB9, 0xFE, 0x67, 0x81, 0xD4, 0x92, 0xF3, 0x39, 0xFD,
	0x45, 0x29, 0xD0, 0x33, 0x65, 0x81, 0x5B, 0xBA, 0xD9, 0xF2, 0x23, 0x36, 0xA3, 0x66, 0xFA, 0x56,
	0x6D, 0x7B, 0x71, 0xE6, 0x4D, 0xBF, 0xAE, 0x5A, 0xFF, 0x6F, 0xA8, 0xD5, 0x92, 0xF2, 0x48, 0xFD,
	0x7F, 0xDD, 0x97, 0xEE, 0x6F, 0xAF, 0x10, 0xB8, 0x6C, 0x60, 0xC6, 0xCE, 0x2F, 0x26, 0xF6, 0x2E,
	0xDF, 0xFB, 0x58, 0x6C, 0x3D, 0xFF, 0xE2, 0x49, 0x46, 0x91, 0x78, 0x66, 0x96, 0xEC, 0x8B, 0xCE,
	0xC1, 0xAD, 0x47, 0xC5, 0xBA, 0x79, 0xD0, 0x59, 0xD6, 0xCC, 0xFD, 0x78, 0x62, 0xAF, 0xFD, 0x45,
	0x47, 0x30, 0x00, 0x5F, 0x61, 0x33, 0x52, 0x3A, 0xCD, 0x6F, 0xE8, 0x3D, 0xD0, 0xB2, 0x48, 0xDC,
	0x65, 0xDD, 0xD7, 0x9A, 0xED, 0xAF, 0xD1, 0xEC, 0xE2, 0xFC, 0xFC, 0x41, 0x25, 0xEE, 0xFD, 0x5D,
	0xAF, 0xB2, 0x7E, 0xC7, 0xE1, 0x4B, 0xCE, 0x81, 0xBF, 0x20, 0xE8, 0x54, 0xB2, 0xE3, 0x48, 0x5C,
	0x65, 0xFF, 0x47, 0xBB, 0xE5, 0xF5, 0xD2, 0x09, 0xC2, 0xD7, 0xE5, 0x0F, 0x08, 0x7A, 0x57, 0x0C,
	0xD0, 0x23, 0xE8, 0x71, 0x29, 0x4A, 0xF5, 0x04, 0x2F, 0xB3, 0xEA, 0xA4, 0xC2, 0x35, 0x6F, 0xA9,
	0xC7, 0x59, 0x8E, 0x98, 0xEC, 0xA7, 0x98, 0x51, 0xFD, 0xE6, 0xFD, 0x2F, 0xAF, 0x53, 0xF4, 0x5A,
	0x28, 0x68, 0x7D, 0xDB, 0xF6, 0xF5, 0xF7, 0x76, 0xA9, 0x6F, 0xF8, 0xFD, 0x7E, 0xF2, 0x48, 0xFE,
	0x65, 0xDD, 0x47, 0xB6, 0xE5, 0xAF, 0xD0, 0xFC, 0xED, 0xEE, 0xF9, 0xF5, 0xAB, 0xEE, 0xFD, 0xF2,
	0x6D, 0xFB, 0x49, 0xF7, 0x5F, 0xFF, 0x5F, 0x63, 0x7F, 0x6F, 0xE9, 0xF5, 0x90, 0xF2, 0x48, 0xFD,
	0x65, 0xDD, 0x47, 0x96, 0xE5, 0xAF, 0xF0, 0xBC, 0xED, 0xEE, 0xF9, 0xFD, 0xAB, 0xEE, 0xFD, 0xF0,
	0xCF, 0xF9, 0x52, 0xF5, 0x17, 0x8B, 0x62, 0x5B, 0xF5, 0x27, 0x6B, 0xD4, 0x33, 0xB7, 0x4D, 0xF9,
	0x65, 0xDD, 0x47, 0x31, 0xEA, 0xA7, 0xD0, 0x3F, 0xED, 0xEE, 0xF9, 0xFD, 0xAB, 0xEF, 0xF5, 0xD2,
	0x5D, 0x5C, 0x5A, 0xFC, 0x9E, 0xFF, 0xE2, 0xC6, 0xFF, 0x6F, 0xE8, 0xD5, 0xB2, 0xF2, 0x48, 0x7D,
	0x64, 0xC5, 0xC7, 0x52, 0xE5, 0xAF, 0xD0, 0x03, 0x80, 0x6E, 0x9B, 0xE1, 0x99, 0xEE, 0xEA, 0x95,
	0x7D, 0xE2, 0x5A, 0x72, 0xD9, 0xBF, 0xE2, 0x80, 0xF7, 0xB9, 0xA8, 0xDD, 0xB2, 0x90, 0x7E, 0xFD,
	0xE5, 0xDD, 0x47, 0x1F, 0xE5, 0xAF, 0xC0, 0x3A, 0x6D, 0xEE, 0xF9, 0xD5, 0xAB, 0xEE, 0xFD, 0x3D,
	0xED, 0xFB, 0xCE, 0xF7, 0x4F, 0xF7, 0xA2, 0x63, 0xFF, 0x6F, 0xE8, 0xD5, 0x92, 0xF2, 0x48, 0xFD,
	0x67, 0x75, 0x47, 0x8F, 0xE5, 0x0F, 0xD0, 0x9C, 0xED, 0xEE, 0x61, 0x1F, 0xAB, 0xEA, 0x66, 0x13,
	0xED, 0x7B, 0x5A, 0xF7, 0x7B, 0xEB, 0xE2, 0x63, 0xFF, 0x6F, 0xE8, 0xD5, 0x92, 0xF2, 0x48, 0x7D,
	0x0F, 0x78, 0x1B, 0xDB, 0x30, 0x4D, 0x9A, 0x8C, 0xED, 0xEE, 0x44, 0xF5, 0xAB, 0xEE, 0x99, 0xB6,
	0x7F, 0xFB, 0x5A, 0xF7, 0x6B, 0xFF, 0xE2, 0x63, 0xEF, 0x6D, 0xE8, 0xD5, 0x92, 0xF2, 0x48, 0xFD,
	0x45, 0xDD, 0x67, 0x66, 0x25, 0x2F, 0xD0, 0x23, 0x6D, 0xEF, 0x9C, 0xF9, 0xAB, 0xFF, 0x2C, 0xF0,
	0x7D, 0xFB, 0x5A, 0xFF, 0x4F, 0xFF, 0xE2, 0x05, 0x13, 0x4E, 0xE8, 0x8B, 0x9C, 0x24, 0x48, 0x0C,
	0x6B, 0xDD, 0xC7, 0x28, 0xF1, 0xAF, 0xD0, 0x8C, 0xED, 0xEE, 0xF9, 0xFF, 0xAB, 0xEE, 0xFD, 0xF2,
	0x15, 0xFB, 0x95, 0xF7, 0x24, 0xFF, 0xA9, 0xEB, 0xFF, 0xF3, 0xE7, 0xDD, 0x92, 0x41, 0xA9, 0xFC,
	0x89, 0xB9, 0x6F, 0x3C, 0x46, 0xEA, 0x49, 0x2C, 0xED, 0xEC, 0xF9, 0xF6, 0xEB, 0x4C, 0xFD, 0xD7,
	0x2A, 0xF4, 0x77, 0x7C, 0x29, 0xED, 0x3B, 0xF3, 0x7F, 0x0A, 0xE8, 0xD4, 0x92, 0x32, 0x48, 0xFD,
	0xFE, 0xCD, 0xE7, 0x0C, 0xF7, 0xA7, 0xD9, 0x8D, 0xA3, 0xEE, 0x71, 0xAC, 0xCB, 0xEE, 0xBC, 0x90,
	0x7D, 0xFF, 0x4A, 0xF7, 0x37, 0xFF, 0xE3, 0x73, 0x28, 0x6F, 0x7E, 0xD5, 0x19, 0xB6, 0x35, 0xFD,
	0xAF, 0x31, 0x23, 0x22, 0xE0, 0xAB, 0x8D, 0xBC, 0xED, 0xEE, 0xF1, 0xF8, 0xAB, 0xEE, 0xFC, 0xDA,
	0x6D, 0xFB, 0x5A, 0x05, 0x4F, 0xFE, 0xE2, 0x66, 0x7E, 0xBA, 0x45, 0xEC, 0x88, 0xD0, 0x11, 0x3C,
	0xBD, 0x4A, 0x70, 0xB6, 0x52, 0x1A, 0xC3, 0xEC, 0xED, 0xEE, 0xB9, 0xFD, 0xAB, 0xEE, 0xF5, 0xF2,
	0x7D, 0x69, 0xFB, 0xF7, 0xED, 0xFF, 0x86, 0x45, 0x8B, 0x6F, 0xA1, 0x97, 0x97, 0xF6, 0x48, 0xEF,
	0x47, 0xDD, 0x47, 0xAC, 0x56, 0xAF, 0xD0, 0x31, 0xED, 0xEE, 0xF9, 0x90, 0xAB, 0xEE, 0xF5, 0xD8,
	0x7D, 0xFB, 0x5A, 0x31, 0x4B, 0xFF, 0xE2, 0x63, 0x27, 0x3B, 0xAF, 0x00, 0x3A, 0xD4, 0x8E, 0xC3,
	0x0B, 0x30, 0x47, 0x92, 0xB9, 0xC6, 0xD0, 0x1C, 0xFD, 0xCC, 0xF9, 0xED, 0xAB, 0xE7, 0xFC, 0xB2,
	0xFD, 0xF3, 0x5A, 0xF7, 0x6B, 0x7D, 0xE2, 0x73, 0x9E, 0x42, 0x21, 0x6B, 0xED, 0x5C, 0x6D, 0xC0
};

static const uint8_t activation_code[BK_AC_SIZE_BYTES] =
{
	0x95, 0x13, 0x12, 0xAD, 0xA1, 0x78, 0x1F, 0x8F, 0x2D, 0xBE, 0x96, 0x4E, 0x72, 0x6C, 0x67, 0xEE,
	0x6A, 0x34, 0xCD, 0x00, 0x9E, 0x62, 0x30, 0x54, 0xCE, 0x5E, 0x3B, 0x4C, 0xCA, 0xA9, 0xD2, 0x22,
	0x52, 0x2E, 0x2A, 0xEC, 0xDD, 0x73, 0xD7, 0x56, 0x97, 0x24, 0x3D, 0x2E, 0x71, 0x07, 0xEE, 0xDE,
	0x2D, 0x83, 0xDE, 0x28, 0x29, 0xB8, 0x10, 0x09, 0xFE, 0x51, 0xAD, 0x63, 0xE4, 0xB9, 0xDB, 0x94,
	0x82, 0x54, 0xC8, 0xAF, 0xE1, 0x70, 0xC2, 0xA5, 0x06, 0x8B, 0x28, 0x04, 0xE4, 0x6F, 0x85, 0x0B,
	0xC2, 0xC2, 0x61, 0x6A, 0xBD, 0x2E, 0x66, 0x88, 0x91, 0x1D, 0x21, 0x6E, 0xE1, 0xA2, 0xE2, 0x39,
	0xC8, 0xC8, 0x3A, 0xB2, 0xC3, 0x50, 0xBE, 0x3E, 0x63, 0xB9, 0xB3, 0xA1, 0x24, 0xCE, 0x22, 0x0B,
	0x7A, 0xAA, 0x28, 0x07, 0x10, 0x78, 0xA7, 0x47, 0x97, 0xC4, 0xDA, 0x30, 0x42, 0x86, 0xE7, 0x90,
	0xB3, 0x29, 0x11, 0xEC, 0x44, 0xE3, 0x94, 0xC4, 0x61, 0x2A, 0xDE, 0xF5, 0xF3, 0xDB, 0x56, 0x19,
	0x10, 0xD3, 0x67, 0x34, 0x4D, 0x0D, 0x42, 0xE0, 0x19, 0xEE, 0x5C, 0x61, 0xDD, 0xFA, 0x16, 0x24,
	0x2A, 0xAE, 0x86, 0xAD, 0x1C, 0xD7, 0x76, 0xAB, 0x3E, 0x36, 0x97, 0x36, 0x24, 0xCE, 0x17, 0xD8,
	0xCE, 0x64, 0x16, 0xA8, 0xBB, 0x0E, 0x67, 0x17, 0xB9, 0x55, 0x6F, 0xB1, 0xF1, 0xDC, 0x18, 0x9C,
	0xD1, 0xC8, 0xBC, 0x9E, 0x18, 0x4E, 0x44, 0x14, 0xEB, 0x3A, 0xB2, 0xE4, 0x6F, 0x54, 0x74, 0xFF,
	0xCD, 0xFE, 0x52, 0x9F, 0x13, 0x0F, 0x25, 0x0B, 0xF7, 0x30, 0xF1, 0xA5, 0xB1, 0x7E, 0xF4, 0x30,
	0x74, 0xAC, 0xE9, 0x86, 0x66, 0xBC, 0x06, 0x8B, 0xDE, 0x57, 0xBF, 0x78, 0x9E, 0x95, 0x80, 0x57,
	0x43, 0x16, 0x24, 0x70, 0xC5, 0xB4, 0xF8, 0xEA, 0x70, 0x96, 0xA9, 0x8C, 0xC8, 0x6B, 0x07, 0xC4,
	0xF5, 0xC2, 0x2C, 0x45, 0x7F, 0x70, 0x6C, 0x8A, 0x5F, 0x37, 0xD0, 0x89, 0xC4, 0xF5, 0x63, 0x01,
	0xB6, 0x21, 0x51, 0x28, 0x4E, 0x0E, 0xED, 0x99, 0x2F, 0x21, 0x4C, 0xB5, 0x5E, 0xDB, 0x07, 0x12,
	0x70, 0x0B, 0xA2, 0xB3, 0x8C, 0x9D, 0x31, 0xF5, 0x7E, 0x77, 0xD2, 0x38, 0x9A, 0x7E, 0x39, 0x53,
	0x7D, 0x84, 0xA1, 0x75, 0x36, 0x2E, 0xBA, 0xA6, 0x7C, 0x6F, 0xA6, 0x3A, 0xD7, 0x35, 0x06, 0xB7,
	0x58, 0x15, 0x6F, 0x4C, 0xC3, 0xFA, 0x87, 0x13, 0xBD, 0x41, 0x28, 0xD0, 0x4D, 0x8A, 0x85, 0x56,
	0x5C, 0xF2, 0x29, 0xDD, 0x3F, 0x00, 0xBF, 0x21, 0xFF, 0x57, 0xE9, 0xC2, 0x62, 0x30, 0xE8, 0x9E,
	0x8C, 0xB4, 0xD5, 0xB1, 0xEF, 0x5B, 0xB8, 0x88, 0xC4, 0xBB, 0x05, 0x0B, 0xE7, 0x2E, 0x31, 0x23,
	0xE3, 0x63, 0x27, 0xCD, 0xAD, 0x28, 0xDC, 0x4A, 0x28, 0xAA, 0xC1, 0x40, 0xBD, 0x84, 0xBC, 0x40,
	0x66, 0x33, 0xAF, 0xA7, 0x36, 0x83, 0x34, 0xF7, 0xD3, 0x76, 0x4C, 0x0C, 0xB1, 0xA1, 0xF6, 0xD6,
	0xB9, 0xDA, 0x53, 0xFF, 0x8D, 0xB2, 0x19, 0xF3, 0xB6, 0xA2, 0xB4, 0xFE, 0x03, 0x47, 0xE9, 0xB2,
	0xDE, 0x0C, 0xED, 0xAF, 0xBC, 0x1B, 0x74, 0xA3, 0x29, 0x36, 0xE8, 0xC3, 0x11, 0xA8, 0xC7, 0x1C,
	0xC7, 0x4E, 0xAD, 0x13, 0xE2, 0x50, 0xEC, 0xF6, 0x1F, 0x1C, 0x5D, 0x93, 0x7C, 0x11, 0x9E, 0xA1,
	0x13, 0x7C, 0xB0, 0x7F, 0x40, 0x38, 0xB3, 0xA2, 0xA2, 0x59, 0xF3, 0x7E, 0x28, 0x62, 0x9D, 0xA1,
	0xE9, 0xC6, 0x1B, 0xB6, 0xE4, 0x55, 0xFF, 0x0D, 0xDC, 0x39, 0xB3, 0x7F, 0xD0, 0x6A, 0xF4, 0x65,
	0x09, 0x59, 0xDC, 0x75, 0x8D, 0x60, 0xD1, 0x87, 0xD9, 0xCF, 0x38, 0x16, 0x42, 0x1E, 0xEF, 0x09,
	0x55, 0x4D, 0x38, 0x1F, 0x30, 0xA6, 0xB8, 0x93, 0xE8, 0x28, 0xE1, 0x74, 0xDA, 0x5D, 0x92, 0x84,
	0xBB, 0x64, 0xD4, 0xB1, 0xD7, 0xB0, 0x9B, 0x8A, 0x49, 0x3F, 0xC5, 0x1A, 0xF7, 0xE1, 0xD2, 0x5A,
	0x1F, 0x2E, 0x5E, 0xE4, 0xAC, 0xF1, 0xDF, 0x3C, 0x34, 0x5B, 0xA8, 0x2A, 0x67, 0x21, 0x67, 0xA6,
	0x7A, 0xE9, 0xEC, 0xC7, 0x48, 0xD4, 0xBB, 0x74, 0x38, 0x3C, 0x6C, 0xFA, 0xE7, 0x34, 0x02, 0x93,
	0xD0, 0x39, 0x81, 0x38, 0xD0, 0x29, 0xBF, 0x56, 0x0D, 0x57, 0xCD, 0x83, 0xCD, 0x77, 0x3E, 0xE6,
	0x00, 0x91, 0x2E, 0xFE, 0xF3, 0xC4, 0xAE, 0x67, 0x8C, 0xB6, 0x3F, 0x46, 0x60, 0x6A, 0xFA, 0x36,
	0x28, 0x94, 0x57, 0xF5, 0xFE, 0xDD, 0xDB, 0x25, 0x35, 0x67, 0x46, 0xA2, 0xB6, 0x5F, 0x01, 0xAC,
	0x95, 0xDF, 0x81, 0xB3, 0x11, 0x5B, 0x90, 0x25, 0x98, 0x9F, 0xC3, 0x22, 0x76, 0x6E, 0xFA, 0x37,
	0x12, 0xB6, 0xB6, 0x04, 0x21, 0x3E, 0xE9, 0x2C, 0x61, 0xF3, 0xE0, 0xFF, 0xFA, 0x54, 0x46, 0x36,
	0x8D, 0x45, 0x7F, 0xEC, 0xB4, 0x55, 0x42, 0x1F, 0x6C, 0x15, 0x48, 0x8B, 0x28, 0xCD, 0x44, 0x87,
	0x11, 0xE0, 0x52, 0xC8, 0xD2, 0x09, 0xCB, 0x08, 0xBD, 0xBE, 0xCB, 0x9D, 0xD3, 0x84, 0xDB, 0x1E,
	0x7A, 0x24, 0x6D, 0x7C, 0xCE, 0x50, 0xCC, 0x7F, 0x8E, 0x63, 0x04, 0x01, 0xAB, 0x4E, 0x90, 0x3A,
	0x4F, 0x27, 0x51, 0x7C, 0x52, 0xF0, 0x05, 0x50, 0x95, 0xCE, 0xC6, 0x3A, 0x83, 0x18, 0xC3, 0x1B,
	0xAD, 0x7F, 0x32, 0x02, 0x70, 0xC6, 0x70, 0x4B, 0x15, 0x05, 0xFE, 0x8A, 0x55, 0x45, 0x7E, 0xCD,
	0x28, 0x20, 0x5A, 0xC5, 0x22, 0x5E, 0xC8, 0xAD, 0x02, 0xB9, 0x22, 0x52, 0x03, 0x0E, 0xD5, 0xBA,
	0x5A, 0xA7, 0xCB, 0xFB, 0x29, 0x89, 0x07, 0x61, 0x63, 0xA1, 0xFC, 0x18, 0x2C, 0xA7, 0xFC, 0xBC,
	0x3D, 0x7C, 0x7C, 0x0E, 0x9F, 0x36, 0x24, 0x2B, 0xEB, 0xB5, 0xFC, 0x7E, 0x4E, 0x83, 0x98, 0x79,
	0x40, 0x24, 0x82, 0xFC, 0x8F, 0x75, 0xF3, 0x18, 0xA4, 0x9E, 0x80, 0xFC, 0x1F, 0x8F, 0x47, 0x6D,
	0x80, 0x67, 0x81, 0x07, 0x3E, 0xB1, 0xC0, 0x93, 0xE4, 0x3E, 0xDE, 0x89, 0x94, 0x95, 0xC5, 0xDE,
	0x95, 0xB6, 0x72, 0xF9, 0xDB, 0xE8, 0xA2, 0x37, 0x95, 0xA4, 0x60, 0x64, 0x78, 0x66, 0x4E, 0x1E,
	0x84, 0xF9, 0xDF, 0x99, 0xEE, 0xC4, 0xDA, 0x46, 0x92, 0x05, 0x2D, 0xE2, 0xFD, 0x42, 0x79, 0x93,
	0x56, 0x98, 0x84, 0x9A, 0x9E, 0x5A, 0x72, 0xD7, 0xE9, 0x58, 0x54, 0xB1, 0xB1, 0x37, 0xD4, 0xC3,
	0x87, 0x26, 0x0A, 0xDC, 0x22, 0x35, 0x44, 0xB2, 0x06, 0xB3, 0x26, 0x6C, 0x71, 0xE5, 0xCF, 0xC7,
	0xC6, 0xF4, 0x34, 0x92, 0xE7, 0x20, 0x3E, 0x93, 0xA9, 0xA0, 0x64, 0x45, 0xDB, 0x19, 0x0F, 0xD3,
	0x2D, 0xD0, 0xAF, 0x1E, 0x44, 0xC0, 0xD4, 0x38, 0xDD, 0xF6, 0xCB, 0x7F, 0xBA, 0xD5, 0xFB, 0x4F,
	0xB9, 0x56, 0x35, 0x0C, 0x0F, 0x09, 0x04, 0x61, 0x80, 0xA9, 0x0A, 0x32, 0x8F, 0x15, 0x43, 0x95,
	0xF7, 0xF8, 0xAF, 0x72, 0x5D, 0x8C, 0x98, 0xFC, 0x5B, 0x96, 0x91, 0x46, 0x92, 0xAB, 0xD1, 0xC2,
	0x32, 0x6E, 0x1F, 0x9D, 0x92, 0x32, 0x76, 0x0C, 0xC6, 0x1D, 0x3D, 0xBB, 0xDB, 0x5B, 0x8F, 0xCB,
	0x60, 0x04, 0x57, 0x96, 0x2C, 0x1A, 0xF6, 0xB9, 0xE2, 0x81, 0x82, 0x80, 0xC8, 0xF3, 0x67, 0x53,
	0x0C, 0x4F, 0x9C, 0x1F, 0x06, 0xF3, 0xC0, 0x16
};
#endif /* BK_KNOWN_ANSWER_TEST */

#ifdef BK_AC_FLASH_DATA
static const uint8_t board_ac[BK_AC_SIZE_BYTES] __attribute__((section("activation_code"))) =
{
	/* TODO : Add a valid AC from the board to be tested into this array for testing */
};
#endif /* BK_AC_FLASH_DATA */

int main()
{
	uint8_t * sram_puf = (uint8_t *)&__puf;
#ifdef BK_KNOWN_ANSWER_TEST
	uint8_t * sram_ac = (uint8_t *)activation_code;
#else
	uint8_t * sram_ac = (uint8_t *)&__ac;
#endif
	int i;

	/* Initialize the GW1NS-9C clocks and peripherals*/
	target_init();

#ifdef BK_KNOWN_ANSWER_TEST
	memcpy(sram_puf, local_sram, BK_SRAM_PUF_SIZE_BYTES);
#endif

	//read data from last 1KB flash and print
	PRINTF("---------- Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd. ----------\r\n");
	PRINTF("PUF Addr : %08X\r\n", (uint32_t)sram_puf);
	PRINTF("AC Addr : %08X\r\n", (uint32_t)sram_ac);

	PRINTF("\r\nRead flash.\r\n");
	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");

	print_hex_string(sram_ac, BK_AC_SIZE_BYTES);

	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");

#ifdef DEBUG_DUMP_PUF_SRAM
	PRINTF("\r\nRead SRAM.\r\n");
	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");

	print_hex_string(sram_puf, BK_SRAM_PUF_SIZE_BYTES);

	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");
#endif /*DEBUG_DUMP_PUF_SRAM */


#ifdef DEBUG_WAIT_LOOP
	{
		volatile unsigned int counter = 0;
		bool test_flag = true;

		PRINTF("\r\nWaiting for debugger trigger!\r\n");

		while (test_flag) {
			counter++;
		}
	}
#endif

	bk_demo_app(sram_puf, sram_ac);	/* BK with no enroll demo */

	while(1);
}
