/*! \copyright Copyright 2018 Intrinsic ID B.V. All rights reserved.\n
    This text contains proprietary, confidential information of Intrinsic ID B.V.,
    and may be used, copied, distributed and/or published only pursuant to the
    terms of a valid license agreement with Intrinsic ID B.V.\n
    This copyright notice must be retained as part of this text at all times.
*/

#include "gw1ns4c.h"

#include "iid_usart.h"
#include "iid_platform.h"

#define PLATFORM_UART          UART0

void iid_usart_setup(uint32_t baudrate)
{
    UART_InitTypeDef UART_InitStruct;

    UART_InitStruct.UART_Mode.UARTMode_Tx = ENABLE;//Tx mode
    UART_InitStruct.UART_Mode.UARTMode_Rx = ENABLE;//Rx mode
    UART_InitStruct.UART_Int.UARTInt_Tx = ENABLE;//Tx interrupt
    UART_InitStruct.UART_Int.UARTInt_Rx = ENABLE;//Rx interrupt
    UART_InitStruct.UART_Ovr.UARTOvr_Tx = DISABLE;
    UART_InitStruct.UART_Ovr.UARTOvr_Rx = DISABLE;
    UART_InitStruct.UART_Hstm = DISABLE;
    UART_InitStruct.UART_BaudRate = baudrate;//Baudrate

    UART_Init(PLATFORM_UART, &UART_InitStruct);
}

//Send
void iid_usart_write(const uint8_t * const data, uint32_t num_bytes)
{
    char * pu8 = (char *)data;

    while (num_bytes--)
    {
        UART_SendChar(PLATFORM_UART, *pu8++);
    }

    /* Wait till the last character has been sent. */
    while(PLATFORM_UART->STATE & UART_STATE_TXBF);
}

//Receive
void iid_usart_read(uint8_t * const data, uint32_t num_bytes)
{
    char * pu8 = (char *)data;

    while (num_bytes--)
    {
        *pu8++ = UART_ReceiveChar(PLATFORM_UART);
    }
}
