
/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2020 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file			main.c
 * @author		Embedded Development Team
 * @version		V1.0.0
 * @date			2020-1-1 09:00:00
 * @brief			Main program body.
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "gw1ns2c.h"
#include "broadkey_demo_app.h"
#include "iid_print.h"
#include "iid_usart.h"

/* Imports ------------------------------------------------------------------*/
extern int __puf;//define in gw1ns2c_flash.ld PUF address
extern int __ac;//define in gw1ns2c_flash.ld AC_FLASH address

/* Macros ------------------------------------------------------------------*/
#define FLASH_AC_BASE		(FLASH_BASE + 0x1FC00)

/* Declaractions ------------------------------------------------------------------*/
static void ReadFlashDetain(uint32_t *data);
static uint8_t hex2char(uint8_t va);
static void print_byte(uint8_t va);


int main()
{
  uint8_t * sram_puf = (uint8_t *)&__puf;
  uint8_t * sram_ac = (uint8_t *)&__ac;

  uint32_t data[256] = {0};
  uint32_t i = 0;
  uint8_t *p = (uint8_t *)data;

  SystemInit();//init system
  iid_usart_setup(115200);//init uart0

  //read data from last 1KB flash and print
  print_string("---------- Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd. ----------\r\n");
  print_string("\r\n00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");
  print_string("Read flash begin.\r\n");

  ReadFlashDetain(data);

  for(i = 0; i < sizeof(data); i++)
  {
	print_byte(*p++);

	if (0 == ((i + 1) % 16))
	{
	  print_string("\r\n");
	}
	else
	{
	  print_string(" ");
	}
  }

  print_string("Read flash end.\r\n");
  print_string("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n\r\n");

  broadkey_demo_app(sram_puf, sram_ac);//broadkey with no enroll demo

  while(1);
}

static uint8_t hex2char(uint8_t va)
{
	if (va <= 9)
	{
		va += '0';
	}
	else
	{
		va += 'A' - 0x0A;
	}

	return va;
}

static void print_byte(uint8_t va)
{
	uint8_t temp;

	temp = hex2char(va >> 4);
	UART_SendChar(UART0, (char)temp);

	temp = hex2char(va & 0x0F);
	UART_SendChar(UART0, (char)temp);
}

static void ReadFlashDetain(uint32_t *data)
{
	uint32_t i = 0;

	for (i = 0; i < 256; i++)
	{
		*(data + i) = *(uint32_t *)(FLASH_AC_BASE + i * 4);
	}
}
